# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Fix all version info in the input Info.plist file.

  % python tweak_info_plist.py --output=out.txt --input=in.txt
      --build_dir=/path/to/xcodebuild --gen_out_dir=/path/to/gen_out_dir
      --keystone_dir=/path/to/KeyStone --version_file=version.txt

See mozc_version.py for the detailed information for version.txt.
"""

__author__ = "mukai"

import datetime
import logging
import optparse
import os
import re

from build_tools import mozc_version

_COPYRIGHT_YEAR = datetime.date.today().year


def _ReplaceVariables(data, environment):
  """Replace all occurrence of the variable in data by the value.

  Args:
    data: the original data string
    environment: a dict which maps from variable names to their values

  Returns:
    the data string which replaces the variables by the value.
  """

  def Replacer(matchobj):
    """The replace function to expand variables.

    Args:
      matchobj: the match object

    Returns:
      The value for the variable if the variable is defined in the
      environment.  Otherwise original string is returned.
    """
    if matchobj.group(1) in environment:
      return environment[matchobj.group(1)]
    return matchobj.group(0)

  return re.sub(r'\$\{(\w+)\}', Replacer, data)


def _RemoveDevOnlyLines(data, build_type):
  """Remove dev-only lines.

  Args:
    data: the original data string
    build_type: build type ("dev" or "stable")

  Returns:
    if build_type == "dev"
      the data string including dev-only lines.
    else:
      the data string excluding dev-only lines.
  """
  pat = re.compile('<!--DEV_ONLY_START-->\n(.*)<!--DEV_ONLY_END-->\n',
                   re.DOTALL)
  if build_type == 'dev':
    return re.sub(pat, r'\1', data)
  else:
    return re.sub(pat, '', data)


def ParseOptions():
  """Parse command line options.

  Returns:
    An options data.
  """
  parser = optparse.OptionParser()
  parser.add_option('--version_file', dest='version_file')
  parser.add_option('--output', dest='output')
  parser.add_option('--input', dest='input')
  parser.add_option('--build_dir', dest='build_dir')
  parser.add_option('--gen_out_dir', dest='gen_out_dir')
  parser.add_option('--auto_updater_dir', dest='auto_updater_dir')
  parser.add_option('--mozc_dir', dest='mozc_dir')
  parser.add_option('--launch_agent_dir', dest='launch_agent_dir')
  parser.add_option('--build_type', dest='build_type')

  (options, unused_args) = parser.parse_args()
  return options


def main():
  """The main function."""
  options = ParseOptions()
  required_flags = [
      'version_file', 'output', 'input', 'build_dir', 'gen_out_dir',
      'auto_updater_dir', 'build_type', 'launch_agent_dir',
  ]
  for flag in required_flags:
    if getattr(options, flag) is None:
      logging.error('--%s is not specified.', flag)
      exit(-1)

  version = mozc_version.MozcVersion(options.version_file)

  copyright_message = '© %d Google Inc.' % _COPYRIGHT_YEAR
  long_version = version.GetVersionString()
  short_version = version.GetVersionInFormat('@MAJOR@.@MINOR@.@BUILD@')
  if options.mozc_dir:
    mozc_dir = options.mozc_dir
  else:
    mozc_dir = os.path.abspath(os.path.join(os.getcwd(), '..'))

  variables = {
      'MOZC_VERSIONINFO_MAJOR':
          version.GetVersionInFormat('@MAJOR@'),
      'MOZC_VERSIONINFO_MINOR':
          version.GetVersionInFormat('@MINOR@'),
      'MOZC_VERSIONINFO_LONG':
          long_version,
      'MOZC_VERSIONINFO_SHORT':
          short_version,
      'MOZC_VERSIONINFO_FINDER':
          'Google Japanese Input %s, %s' % (long_version, copyright_message),
      'GEN_OUT_DIR':
          os.path.abspath(options.gen_out_dir),
      'BUILD_DIR':
          os.path.abspath(options.build_dir),
      'AUTO_UPDATER_DIR':
          os.path.abspath(options.auto_updater_dir),
      'MOZC_DIR':
          mozc_dir,
      'LAUNCH_AGENT_DIR':
          os.path.abspath(options.launch_agent_dir),
  }

  with open(options.input, encoding='utf-8') as input_file:
    with open(options.output, 'w', encoding='utf-8') as output_file:
      output_file.write(
          _RemoveDevOnlyLines(_ReplaceVariables(input_file.read(), variables),
                              options.build_type))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Build an installer for macOS (.pkg file).

This script creates a .pkgproj file with arguments and build an installer.
"""

import argparse
import logging
import os
import shutil
import sys
import tempfile

from build_tools import util


def ParseArguments():
  """Parses command line options."""
  parser = argparse.ArgumentParser()
  parser.add_argument('--input')
  parser.add_argument('--output')
  parser.add_argument('--pkgproj_command')
  parser.add_argument('--pkgproj_input')
  parser.add_argument('--pkgproj_output')
  parser.add_argument('--version_file')
  parser.add_argument('--build_type')
  parser.add_argument('--auto_updater_dir')
  return parser.parse_args()


def CopyFiles(input_files, out_dir):
  for src in input_files:
    if src.endswith('.zip'):
      util.ExtractZip(src, out_dir)
    else:
      shutil.copyfile(src, os.path.join(out_dir, os.path.basename(src)))


def TweakPkgproj(args, src_dir):
  """Creates a .pkgproj file by filling the arguments."""
  commands = [args.pkgproj_command,
              '--output', args.pkgproj_output,
              '--input', args.pkgproj_input,
              '--version_file', args.version_file,
              '--gen_out_dir', src_dir,
              '--build_dir', src_dir,
              '--auto_updater_dir', args.auto_updater_dir,
              '--mozc_dir', os.getcwd(),
              '--launch_agent_dir', src_dir,
              '--build_type', args.build_type,
              ]
  util.RunOrDie(commands)


def BuildInstaller(args, out_dir):
  """Creates a .pkg file by running packagesbuild."""
  # Make sure Packages is installed
  packagesbuild_path = ''
  if os.path.exists('/usr/local/bin/packagesbuild'):
    packagesbuild_path = '/usr/local/bin/packagesbuild'
  elif os.path.exists('/usr/bin/packagesbuild'):
    packagesbuild_path = '/usr/bin/packagesbuild'
  else:
    logging.critical('error: Cannot find "packagesbuild"')
    sys.exit(1)

  # Build the package
  commands = [
      packagesbuild_path, args.pkgproj_output, '--build-folder', out_dir
  ]
  util.RunOrDie(commands)

  shutil.copyfile(os.path.join(out_dir, 'Mozc.pkg'), args.output)


def main():
  args = ParseArguments()

  with tempfile.TemporaryDirectory() as tmp_dir:
    util.ExtractZip(args.input, tmp_dir)
    TweakPkgproj(args, os.path.join(tmp_dir, 'installer'))
    BuildInstaller(args, tmp_dir)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A tool to generate aux_dictionary.txt from aux_dictionary.tsv.

* Create a dictionary with entries of key and value in the TSV file.
  + Fields: [key, value, base_key, base_value, cost_offset]
* lid, rid and cost are copied from entries of base_key and base_value.
* cost_offset adjusts the cost copied from the base entry. If the cost_offset
  is -1, the new entry is ranked higher than the base entry.

gen_aux_dictionary.py --output aux_dictionary.txt
--aux_tsv data/oss/aux_dictionary.tsv
--dictionary_txts data/dictionary_oss/dictionary0*.txt
"""

import argparse


class Dictionary():
  """Class for dictionary0*.txt."""

  def __init__(self):
    self.entry_set = set()
    self.data = {}

  def Parse(self, dictionary_txts):
    for file in dictionary_txts:
      self._ParseFile(file)

  def _ParseFile(self, file):
    """Parses the file and set values to data and entry_set."""
    for line in open(file, encoding='utf-8'):
      key, lid, rid, cost, value, *_ = line.rstrip().split('\t')

      # Update data
      data_key = '\t'.join([key, value])
      data_value = [lid, rid, cost]
      self.data.setdefault(data_key, []).append(data_value)

      cost = int(cost)

      # Update entry_set
      entry_key = '\t'.join([lid, rid, key, value])
      self.entry_set.add(entry_key)

  def Exists(self, key, value, lid, rid):
    entry_key = '\t'.join([lid, rid, key, value])
    return entry_key in self.entry_set

  def GetDataList(self, key, value):
    data_key = '\t'.join([key, value])
    return self.data[data_key]


class AuxDictionary():
  """Class for aux_dictionary.tsv."""

  def __init__(self, dictionary):
    self.dictionary = dictionary
    self.aux_list = []

  def Parse(self, aux_tsv):
    """Parses the file and update aux_list."""
    for line in open(aux_tsv, encoding='utf-8'):
      if line.startswith('#'):
        continue
      key, value, base_key, base_value, cost_offset = line.rstrip().split('\t')
      for data in self.dictionary.GetDataList(base_key, base_value):
        base_lid, base_rid, base_cost = data
        if self.dictionary.Exists(key, value, base_lid, base_rid):
          # Not overwrite the existing entry.
          continue
        cost = int(base_cost) + int(cost_offset)
        self.aux_list.append([key, base_lid, base_rid, str(cost), value])

  def WriteFile(self, output):
    with open(output, 'w', encoding='utf-8') as file:
      for aux_entry in self.aux_list:
        file.write('\t'.join(aux_entry) + '\n')


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument('--aux_tsv')
  parser.add_argument('--dictionary_txts', nargs='+')
  parser.add_argument('--output')
  args = parser.parse_args()

  dictionary = Dictionary()
  dictionary.Parse(args.dictionary_txts)

  aux = AuxDictionary(dictionary)
  aux.Parse(args.aux_tsv)
  aux.WriteFile(args.output)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

__author__ = "taku"

import codecs
import itertools
import optparse
import re
import sys


# We use 2-bits bitmap data for JISX0208, JISX0212 and JISX0213 clustering.
# For remaining categories (i.e. ASCII, CP932, JISX0201 and UNICODE_ONLY),
# we introduce heuristics to check them and use '00' bits for all of them.
CATEGORY_BITMAP = {
    'JISX0208': 1,
    'JISX0212': 2,
    'JISX0213': 3,
}


def IsValidUCS2(n):
  """Returns True if the n is valid code in UCS2."""
  return 0 <= n <= 0xFFFF


def IsValidUCS4(n):
  """Returns True if the n is valid code in UCS4."""
  return 0 <= n <= 0x7FFFFFFF


class CodePointCategorizer(object):
  """Categorizer of ucs4 code points."""

  _UCS2_PATTERN = re.compile(r'^0x([0-9A-F]{4})')

  # UCS4 pattern supports only JIS X 0213.
  # Note that Some JIS X 0213 characters are described as 'U+xxxx+xxxx',
  # and this pattern ignores latter +xxxx part intentionally.
  _UCS4_PATTERN = re.compile(r'^U\+([0-9A-F]+)')

  def __init__(self, jisx0201file, jisx0208file):
    self._jisx0201 = CodePointCategorizer._LoadJISX0201(jisx0201file)
    self._jisx0208 = CodePointCategorizer._LoadJISX0208(jisx0208file)
    self._exceptions = CodePointCategorizer._LoadExceptions()

    # Make a list of code point tables in the priority order.
    self._table_list = [
        ('JISX0208', self._exceptions),  # Vender specific code.
        ('JISX0201', self._jisx0201),
        ('JISX0208', self._jisx0208),
    ]

  @staticmethod
  def _LoadTable(filename, column_index, pattern, validater):
    result = set()
    for line in codecs.open(filename, encoding='utf-8'):
      if line.startswith('#'):
        # Skip a comment line.
        continue

      columns = line.split()
      match = pattern.search(columns[column_index])
      if match:
        ucs = int(match.group(1), 16)
        if validater(ucs):
          result.add(ucs)

    return result

  @staticmethod
  def _LoadJISX0201(filename):
    return CodePointCategorizer._LoadTable(
        filename, 1, CodePointCategorizer._UCS2_PATTERN, IsValidUCS2)

  @staticmethod
  def _LoadJISX0208(filename):
    result = CodePointCategorizer._LoadTable(
        filename, 2, CodePointCategorizer._UCS2_PATTERN, IsValidUCS2)
    result.update([
        0xFF3C,  # (FULLWIDTH REVERSE SOLIDS) should be in JISX0208
        0xFF0D,  # (FULLWIDTH HYPHEN MINUS) should be in JISX0208
        ])
    return result

  # The following chars have different mapping in
  # Windows and Mac. Technically, they are platform dependent
  # characters, but Mozc treat them so that they are normal characters
  # defined in JISX0208
  @staticmethod
  def _LoadExceptions():
    # treat Unicode Japanese incompatible characters as JISX0208.
    return set([
        0x00A5,  # YEN SIGN
        0x203E,  # OVERLINE
        0x301C,  # WAVE DASH
        0xFF5E,  # FULL WIDTH TILDE
        0x2016,  # DOUBLE VERTICAL LINE
        0x2225,  # PARALLEL TO
        0x2212,  # MINUS SIGN
        0xFF0D,  # FULL WIDTH HYPHEN MINUS
        0x00A2,  # CENT SIGN
        0xFFE0,  # FULL WIDTH CENT SIGN
        0x00A3,  # POUND SIGN
        0xFFE1,  # FULL WIDTH POUND SIGN
        0x00AC,  # NOT SIGN
        0xFFE2,  # FULL WIDTH NOT SIGN
        ])

  def GetCategory(self, codepoint):
    """Returns category name of the codepoint, or None for invalid input."""
    if not IsValidUCS4(codepoint):
      return None

    # Special handling for ascii code point.
    if codepoint <= 0x007F:
      return 'ASCII'

    # Then look for loaded table list in the order.
    for name, table in self._table_list:
      if codepoint in table:
        return name

    # Not found in any tables, so return "UNICODE_ONLY" as a fallback.
    return 'UNICODE_ONLY'

  def MaxCodePoint(self):
    """Returns the max of code points in the loaded table."""
    return max(max(table) for _, table in self._table_list)


def GroupConsecutiveCodepoints(codepoint_list):
  """Takes sorted codepoint list and groups by consecutive code points."""
  result = []

  prev = None
  current = []
  for codepoint in codepoint_list:
    if prev is not None and codepoint != prev + 1:
      result.append(current)
      current = []
    current.append(codepoint)
    prev = codepoint

  if current:
    result.append(current)

  return result


def ParseOptions():
  """Parses command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--jisx0201file', dest='jisx0201file',
                    help='File path for the unicode\'s JIS0201.TXT file')
  parser.add_option('--jisx0208file', dest='jisx0208file',
                    help='File path for the unicode\'s JIS0208.TXT file')
  parser.add_option('--output', dest='output',
                    help='output file path. If not specified, '
                    'output to stdout.')

  return parser.parse_args()[0]


def GenerateJisX0208Bitmap(category_list, name):
  r"""Generats bitmap data code.

  The generated data looks something like:
  namespace {
  const uint64_t nameIndex = 0xXXXXXXXXXXXX;
  const uint32_t name[NN][32] = {
    {  // 0000 - 03FF
      0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX,
      0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX,
      ...
      0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX,
    },
    {  // 0400 - 07FF
      0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX, 0xXXXXXXXX,
      ...
    },
  };
  }  // namespace

  Args:
    category_list: a list of categories.
    name: a bitmap name.
  """
  lines = []

  # Create bitmap list.
  # Encode each code point category to 1-bits.
  # The result should be uint32_t, so group each consecutive
  # (at most) 32 code points.
  bit_list = []
  for _, group in itertools.groupby(enumerate(category_list),
                                    lambda args: args[0] // 32):  # codepoint/32
    # Fill bits from LSB to MSB for each group.
    bits = 0
    for index, (_, category) in enumerate(group):
      bit = 1 if CATEGORY_BITMAP.get(category, 0) else 0
      bits |= bit << index
    bit_list.append(bits)

  index = 0
  data = []
  data_size = 0
  # Group by 1024 characters (= 128 bit_list units).
  for key, group1 in itertools.groupby(enumerate(bit_list),
                                       lambda args: args[0] // (1024 // 32)):
    block = ['  {  // U+%04X - U+%04X\n' % (key * 1024, (key + 1) * 1024 - 1)]
    has_value = False

    # Output the content. Each line would have (at most) 4 integers
    for _, group2 in itertools.groupby(group1,
                                       lambda args: args[0] // 4):  # index/4
      line = []
      for _, bits in group2:
        line.append('0x%08X' % bits)
        if bits > 0:
          has_value = True
      block.append('    %s,\n' % ', '.join(line))
    block.append('  },\n')

    if has_value:
      index += (1 << key)
      data_size += 1
      data.extend(block)

  lines.extend(['namespace {\n',
                'const uint64_t %sIndex = 0x%016X;\n' % (name, index),
                'const uint32_t %s[%d][32] = {\n' % (name, data_size)])
  lines.extend(data)
  lines.extend(['};\n',
                '}  // namespace\n'])

  return lines


def GenerateCharacterSetHeader(category_list):
  """Generates lines of character_set.h file."""
  bitmap_name = 'kJisX0208Bitmap'
  bitmap_size = 65536

  lines = []

  # File header comments.
  lines.extend(['// This file is generated by base/gen_character_set.py\n',
                '// Do not edit me!\n',
                '\n'])

  # We use bitmap to check JISX0208 for code point 0 to 65535.
  lines.extend(GenerateJisX0208Bitmap(category_list[:bitmap_size], bitmap_name))
  lines.append('\n')

  return lines


def main():
  options = ParseOptions()

  # Generates lines of the header file.
  categorizer = CodePointCategorizer(options.jisx0201file, options.jisx0208file)
  category_list = [
      categorizer.GetCategory(codepoint)
      for codepoint in range(categorizer.MaxCodePoint() + 1)]
  generated_character_set_header = GenerateCharacterSetHeader(category_list)

  # Write the result.
  if options.output:
    output = open(options.output, 'w')
    try:
      output.writelines(generated_character_set_header)
    finally:
      output.close()
  else:
    sys.stdout.writelines(generated_character_set_header)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A tool to generate test sentences for stress test."""

import argparse


def GenerateHeader(input_file, output_file):
  """Convert the file to C++ code."""
  output = []
  with open(input_file, encoding='utf-8') as f:
    for line in f:
      if line.startswith('#'):
        continue
      items = line.rstrip('\r\n').split('\t')
      if len(items) > 1:
        line = items[1]
      else:
        line = items[0]
      if not line:
        continue
      output.append(line)

  with open(output_file, 'w', encoding='utf-8') as f:
    f.write('const char *kTestSentences[] = {\n')
    for line in output:
      escaped = line.replace('\\', '\\\\').replace('"', '\\"')
      f.write(' "%s",\n' % escaped)
    f.write('};\n')


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument('--input')
  parser.add_argument('--output')
  args = parser.parse_args()
  GenerateHeader(args.input, args.output)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Create a zip file from the input files.

If an input file is a zip file, the zip file is extracted.

% python3 zip_file.py --inputs a.zip b.txt --output c.zip

In the above case, c.zip contains extracted files of a.zip, but not a.zip.
"""

import argparse
import os
import shutil
import tempfile

from build_tools import util


def ParseArguments():
  parser = argparse.ArgumentParser()
  parser.add_argument('--inputs', nargs='+')
  parser.add_argument('--output')
  return parser.parse_args()


def CopyFiles(input_files, out_dir):
  for src in input_files:
    if src.endswith('.zip'):
      util.ExtractZip(src, out_dir)
    else:
      shutil.copyfile(src, os.path.join(out_dir, os.path.basename(src)))


def main():
  args = ParseArguments()

  with tempfile.TemporaryDirectory() as tmp_dir:
    out_path = 'installer'
    out_dir = os.path.join(tmp_dir, out_path)
    os.mkdir(out_dir)

    CopyFiles(args.inputs, out_dir)
    basename = os.path.splitext(args.output)[0]
    shutil.make_archive(basename, format='zip', root_dir=tmp_dir)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tool to update quality regression test result files.

quality_regression_main --test_file query.tsv > result.tsv
quality_regression.py --input result.tsv --input_version 4400
  --output quality_regression.tsv --base quality_regression.tsv
"""

import argparse
from build_tools import mozc_version


def LoadBase(base_file):
  """Load the base file to store the last modified version."""
  base_data = {}
  with open(base_file, encoding='utf-8') as file:
    for line in file:
      if line.startswith('#'):
        continue
      items = line.strip('\n').split('\t')
      result = '\t'.join(items[0:5])  # status, input, output, command, argument
      version = items[5]
      base_data[result] = version
  return base_data


def LoadInput(input_file, input_version, base_data):
  """Load the input file generated by quality_regression_main."""
  output = []
  with open(input_file, encoding='utf-8') as file:
    for line in file:
      line = line.strip('\n')
      if line.startswith('#'):
        output.append(line)
        continue
      items = line.split('\t')
      result = '\t'.join(items[0:5])  # status, input, output, command, argument
      version = base_data.get(result, input_version)
      output.append(result + '\t' + version)
  return output


def SaveOutput(output_file, output_lines):
  """Save the output file."""
  fields = ['status', 'input', 'output', 'command', 'argument', 'version']
  with open(output_file, 'w', encoding='utf-8') as file:
    file.write('# ' + '\t'.join(fields) + '\n')
    file.write('\n'.join(output_lines) + '\n')


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument('--base')
  parser.add_argument('--input')
  parser.add_argument('--version_file')
  parser.add_argument('--output')
  parser.add_argument('--data_type')
  args = parser.parse_args()

  if args.version_file:
    version = mozc_version.MozcVersion(args.version_file)
    use_build_oss = (args.data_type == 'oss')
    version_string = version.GetShortVersionString(use_build_oss=use_build_oss)
  else:
    version_string = ''

  if args.base:
    base_data = LoadBase(args.base)
  else:
    base_data = {}

  output_lines = LoadInput(args.input, version_string, base_data)
  SaveOutput(args.output, output_lines)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2021, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Script to rename icons files and pack them into a zip file.

build_icon.py --inputs ui-tool.png hiragana.svg ... --output icons.zip
"""

import argparse
import os
import shutil
import tempfile

from xml.dom import minidom

RENAME_RULES = {
    'product_icon_32bpp-128.png': 'mozc.png',
    'icon.svg': 'mozc.svg',
    'full_ascii.svg': 'alpha_full.svg',
    'full_katakana.svg': 'katakana_full.svg',
    'half_ascii.svg': 'alpha_half.svg',
    'half_katakana.svg': 'katakana_half.svg',
    'ui-dictionary.png': 'dictionary.png',
    'ui-properties.png': 'properties.png',
    'ui-tool.png': 'tool.png',
}

VARIANT_SVGS = [
    'dictionary.svg',
    'properties.svg',
    'tool.svg',
]

VARIANT_ATTRIBUTES = {
    'white': {
        'fill': 'white',
    },
    'outlined': {
        'stroke': 'lightgray',
        'stroke-width': '0.9',
    },
}


def ParseArguments():
  parser = argparse.ArgumentParser()
  parser.add_argument('--inputs', nargs='+')
  parser.add_argument('--output')
  return parser.parse_args()


def GetDestName(src) -> str:
  basename = os.path.basename(src)
  return RENAME_RULES.get(basename, basename)


def ModifySvg(base_svgfile, new_svgfile, attributes):
  with minidom.parse(base_svgfile) as doc:
    for path in doc.getElementsByTagName('path'):
      if path.hasAttribute('fill'):
        continue
      for key, value in attributes.items():
        path.setAttribute(key, value)
    with open(new_svgfile, 'w') as output:
      doc.writexml(output)


def CreateVariantIcons(base_svgfile, variant_attributes):
  dirname = os.path.dirname(base_svgfile)
  basename = os.path.basename(base_svgfile)

  for name, attributes in variant_attributes.items():
    variant_dir = os.path.join(dirname, name)
    os.makedirs(variant_dir, exist_ok=True)
    ModifySvg(base_svgfile, os.path.join(variant_dir, basename), attributes)


def main():
  args = ParseArguments()
  with tempfile.TemporaryDirectory() as tmp_dir:
    for src in args.inputs:
      dest_name = GetDestName(src)
      dest_path = os.path.join(tmp_dir, dest_name)
      shutil.copyfile(src, dest_path)
      if dest_name in VARIANT_SVGS:
        CreateVariantIcons(dest_path, VARIANT_ATTRIBUTES)
    basename = os.path.splitext(args.output)[0]
    shutil.make_archive(basename, format='zip', root_dir=tmp_dir)


if __name__ == '__main__':
  main()
